"""Django forms for the socialprofile application"""
from django import forms
from django.contrib.auth.models import User
from models import SocialProfile
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _
from widgets import H5EmailInput
import logging

LOGGER = logging.getLogger(name='socialprofile')

class SocialProfileForm(forms.ModelForm):
    """Master form for editing the user's profile"""

    username = forms.CharField(max_length=30, label='User Name')
    email = forms.EmailField(label="Email Address", widget=H5EmailInput())
    first_name = forms.CharField(max_length=30, required=False, label='First Name', )
    last_name = forms.CharField(max_length=30, required=False, label='Last Name')
    returnTo = forms.CharField(widget=forms.HiddenInput, required=False, initial='/') #URI to Return to after save

    class Meta():
        """Configuration for the ModelForm"""
        model = SocialProfile
        exclude = {'user'} #Don't let through for security reasons, user should be based on logged in user only

    def clean_description(self):
        """Automatically called by Django, this method 'cleans' the description, in our case stripping HTML out of description"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean_description")

        return strip_tags(self.cleaned_data['description'])

    def clean(self):
        """Automatically called by Django, this method 'cleans' the whole form"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean")

        changed_data = self.changed_data

        user_field_objects = User._meta.fields

        user_fields = []
        for field_object in user_field_objects:
            user_fields.append(field_object.name)

        user_dirty = False

        for changed_field in changed_data:
            if changed_field in user_fields:
                user_dirty = True
                setattr(self.instance.user, changed_field, self.cleaned_data.get(changed_field))

            if changed_field == 'username': # Check Username for Uniqueness
                try:
                    User.objects.get(username=self.cleaned_data.get('username'))
                    raise forms.ValidationError([_("Your new username is not available!")])
                except ObjectDoesNotExist:
                    pass # good news, the new username is available

            if user_dirty:
                self.instance.user.save()

        return self.cleaned_data


"""Django Models for SocialProfile App"""

# pylint: disable=W0613, W0141

from django.db.models.signals import post_save
from social_auth.backends.facebook import FacebookBackend
from social_auth.backends.google import GoogleOAuth2Backend
from social_auth.backends.twitter import TwitterBackend
from django.db import models
from django.contrib.auth.models import User
from social_auth.signals import socialauth_registered, pre_update
from urllib import urlencode
from urllib2 import Request, urlopen
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
import logging

LOGGER = logging.getLogger(name='socialprofile.models')

class SocialProfile(models.Model):
    """Main SocialProfile Object - Holds extra profile data retrieved from auth providers"""
    GENDER_CHOICES = (
        (_('male'), _('Male')),
        (_('female'), _('Female')),
        (_('other'), _('Other')),
        ('', '')
        )
    user = models.OneToOneField(User, related_name='social_profile', verbose_name=_("Social Profile"))
    gender = models.CharField(max_length=10, blank=True, choices=GENDER_CHOICES, verbose_name=_("Gender"))
    url = models.URLField(blank=True, verbose_name=_("Homepage"), help_text=_("Where can we find out more about you?"))
    image_url = models.URLField(blank=True, verbose_name=_("Avatar Picture"))
    description = models.TextField(blank=True, verbose_name=_("Description"), help_text=_("Tell us about yourself!"))

    class Meta:
        verbose_name = _("Social Profile")
        verbose_name_plural = _("Social Profiles")
        ordering = ['user__username']

    def __unicode__(self):
        return self.user.username

    @models.permalink
    def get_absolute_url(self):
        return 'sp_profile_other_view_page', [self.user.username]


def create_user_profile(sender, instance, created, **kwargs):
    """Creates a UserProfile Object Whenever a User Object is Created"""
    if created:
        SocialProfile.objects.create(user=instance)

post_save.connect(create_user_profile, sender=User)

def facebook_extra_values(sender, user, response, details, **kwargs):
    """Populates a UserProfile Object when a new User is created via Facebook Auth"""
    user.last_name = response.get('last_name', '')
    user.first_name = response.get('first_name', '')
    profile = user.social_profile
    profile.gender = response.get('gender', '')
    if response.get('username') is not None:
        profile.image_url = 'https://graph.facebook.com/' + response.get('username') + '/picture'
        profile.url = 'http://facebook.com/' + response.get('username')
    profile.url = response.get('link', '')
    if response.get('hometown') is not None:
        profile.description = response.get('hometown').get('name')

    profile.save()

    return True

socialauth_registered.connect(facebook_extra_values, sender=FacebookBackend)

def google_extra_values(sender, user, response, details, **kwargs):
    """Populates a UserProfile Object when a new User is created via Google Auth"""
    LOGGER.debug('socialprofile.models.google_extra_values')
    user_info_url = "https://www.googleapis.com/oauth2/v1/userinfo"

    data = {'access_token': response.get('access_token', ''), 'alt': 'json'}
    params = urlencode(data)
    try:
        request = Request(user_info_url + '?' + params, headers={'Authorization': params})
        result = simplejson.loads(urlopen(request).read())

        user.last_name = result.get('family_name', '')
        user.first_name = result.get('given_name', '')
        profile = user.social_profile
        profile.gender = result.get('gender', '')
        profile.image_url = result.get('picture', '')
        profile.url = result.get('link', '')

        profile.save()
    except:
        pass

    return True

socialauth_registered.connect(google_extra_values, sender=GoogleOAuth2Backend)

def twitter_extra_values(sender, user, response, details, **kwargs):
    """Populates a UserProfile Object when a new User is created via Twitter Auth"""
    try:
        first_name, last_name = response.get('name', '').split(' ', 1)
    except:
        first_name = response.get('name', '')
        last_name = ''
    user.last_name = last_name
    user.first_name = first_name
    profile = user.social_profile
    if response.get('screen_name') is not None:
        profile.url = 'http://twitter.com/' + response.get('screen_name', '')
    profile.image_url = response.get('profile_image_url_https', '')
    profile.description = response.get('description', '')

    profile.save()

    return True

socialauth_registered.connect(twitter_extra_values, sender=TwitterBackend)

def update_user_details(backend, details, response, user, is_new=False, *args,
                        **kwargs):
    """Override of social_auth method, to prevent details from getting updated."""
    LOGGER.debug("socialprofile.models.update_user_details")
    changed = False  # flag to track changes

    signal_response = lambda (receiver, response): response
    signal_kwargs = {'sender': backend.__class__, 'user': user,
                     'response': response, 'details': details}

    changed |= any(filter(signal_response, pre_update.send(**signal_kwargs)))

    # Fire socialauth_registered signal on new user registration
    if is_new:
        changed |= any(filter(signal_response,
            socialauth_registered.send(**signal_kwargs)))

    if changed:
        user.save()


"""Django Views for the socialprofile module"""

# pylint: disable=R0901

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.http import Http404
from django.views.generic import TemplateView, FormView, UpdateView, DeleteView
from models import SocialProfile
from django.forms.models import model_to_dict
from django.utils.translation import ugettext_lazy as _

from forms import SocialProfileForm

import logging

LOGGER = logging.getLogger(name='socialprofile')

DEFAULT_RETURNTO_PATH = getattr(settings, 'DEFAULT_RETURNTO_PATH', '/')

class SelectAuthView(TemplateView):
    """
    Lets users choose how they want to request access.

    url: /select
    """
    template_name = 'socialprofile/sp_account_select.html'

    def get_context_data(self, **kwargs):
        """Ensure that 'next' gets passed along"""
        LOGGER.debug('socialprofile.views.SelectAuthView.get_context_data')

        nextPage = self.request.GET.get(REDIRECT_FIELD_NAME, DEFAULT_RETURNTO_PATH)

        context = super(SelectAuthView, self).get_context_data(**kwargs)
        context['next_param'] = REDIRECT_FIELD_NAME
        context['next_url'] = nextPage
        return context


class SocialProfileView(FormView):
    """
    Profile View Page

    url: /profile/view
    """
    template_name = 'socialprofile/sp_profile_view.html'

    form_class = SocialProfileForm

    http_method_names = ['get'] #Limit to get for security reasons

    def get_initial(self):
        """Load up the default data to show in the display form."""
        LOGGER.debug("socialprofile.views.SocialProfileView.get_initial")
        username = self.kwargs.get('username')
        if username:
            user = get_object_or_404(User, username=username)
        elif self.request.user.is_authenticated():
            user = self.request.user
        else:
            raise Http404 #Case where user gets to this view anonymously for non-existent user

        returnTo = self.request.GET.get('returnTo', DEFAULT_RETURNTO_PATH)

        social_profile = SocialProfile.objects.get(user=user)

        self.object = social_profile

        initial_data = model_to_dict(social_profile)
        initial_data.update(model_to_dict(user))
        initial_data.update({'returnTo': returnTo})

        return initial_data


class SocialProfileEditView(UpdateView):
    """
    Profile Editing View

    url: /profile/edit
    """

    template_name = 'socialprofile/sp_profile_edit.html'

    form_class = SocialProfileForm

    model = SocialProfile

    def get_object(self, queryset=None):
        return SocialProfile.objects.get(user=self.request.user) #Force get from current user for security

    def get_initial(self):
        """Load up the default data to show in the form."""
        LOGGER.debug("socialprofile.views.SocialProfileEditView.get_initial")

        returnTo = self.request.GET.get('returnTo', DEFAULT_RETURNTO_PATH)
        self.success_url = returnTo

        initial_data = self.initial.copy() #Copy data loaded automatically to start with
        initial_data.update(model_to_dict(self.request.user)) # Add current user data
        initial_data.update({'returnTo': returnTo})

        return initial_data

    def form_valid(self, form):
        messages.add_message(self.request, messages.INFO, _('Your profile has been updated.'))
        self.success_url = form.cleaned_data.get('returnTo')
        return super(SocialProfileEditView, self).form_valid(form)

    def form_invalid(self, form):
        messages.add_message(self.request, messages.INFO, _('Your profile has NOT been updated.'))
        return super(SocialProfileEditView, self).form_invalid(form)


class DeleteSocialProfileView(DeleteView):
    """
    Account Delete Confirm Modal View

    url: /delete
    """

    success_url = reverse_lazy('sp_logout_page')

    template_name = "socialprofile/sp_delete_account_modal.html"

    model = User

    def get_object(self, queryset=None):
        """Get the object that we are going to delete"""
        return self.request.user



"""Custom Widget to enable use of HTML5 Form element types"""
from django.forms.widgets import TextInput, DateInput, DateTimeInput, TimeInput

class H5EmailInput(TextInput):
    """HTML5 Email Input Type"""
    input_type = 'email'

class H5NumberInput(TextInput):
    """HTML5 Number Input Type"""
    input_type = 'number'

class H5TelephoneInput(TextInput):
    """HTML5 Telephone Number Input Type"""
    input_type = 'tel'

class H5DateInput(DateInput):
    """HTML5 Date Input Type"""
    input_type = 'date'

class H5DateTimeInput(DateTimeInput):
    """HTML5 DateTime Input Type"""
    input_type = 'datetime'

class H5TimeInput(TimeInput):
    """HTML5 Time Input Type"""
    input_type = 'time'


from setuptools import setup, find_packages

setup(
    name="django-socialprofile",
    version="0.1.6",
    url='http://timlwhite.com',
    license='BSD',
    description="django-socialprofile enables users to manage their user profile built by logging in via a social service such as Google, Twitter, or Facebook. It relies on django-socialauth."
    ,
    long_description=open('README.rst').read(),

    author='Tim White',
    author_email='tim@cyface.com',

    packages=['socialprofile',
              'socialprofile.templatetags',
              ],
    include_package_data=True,
    zip_safe=True,

    install_requires=['django>=1.4',
                      'django-social-auth>=0.7.0', ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        ]
)
